{
 Country - The national application support for DOS.
 Copyright (C) 1998-2002  Henrich Fukna <fuky@azet.sk>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}
program Country;
uses Dos_Intl, Lib;

const
  Copyright: String = 'Country';         { identity string }
  Version = '2.1.0';                     { version string constant }
  ExeName = 'COUNTRY';                   { project name }

                                         { currency formats }
  sCurrencyFormat: array[0..4] of String = (
                   ^B'1'^C'234'^D^E'0',  { $123.00 }
                   '1'^C'234'^D^E'0'^B,  { 123.00$ }
                   ^B' 1'^C'234'^D^E'0', { $ 123.00 }
                   '1'^C'234'^D^E'0 '^B, { 123.00 $ }
                   '1'^C'234'^B^D^E'0'); { 123$00 }

                                         { date formats }
  sDateFormat: array[0..2] of String = (
                   'MM'^A'DD'^A'YY',     { USA }
                   'DD'^A'MM'^A'YY',     { Europe }
                   'YY'^A'MM'^A'DD');    { Japan }

                                         { time formats }
  sTimeFormat: array[0..1] of String = (
                   'hh'^F'mm'^F'ss tt',  { 12 hour clock }
                   'hh'^F'mm'^F'ss');    { 24 hour clock }

{*****************************************************************}

{*
 * Gets the formated string from array of format strings.
 * The format control chars:
 *  ^A means date separator
 *  ^B means national currency string
 *  ^C means thousands separator
 *  ^D means decimals separator
 *  ^E means how many times is the next char copied
 *  ^F means time separator
 *
 * @param N                     index to the string format array
 * @param M                     high index of string format array
 * @param P                     string format array
 * @return formated string
 *}
function GetFmtStr(N, M: Word; P: array of String): String;
var
  I, J: Byte;
  S: String;
begin
  S:= '';
  if N in [0..M] then begin              { check index range }
    for I:= 1 to Length(P[N]) do begin   { for all chars in the format }
      case P[N, I] of                    { string replace control chars }
        ^A:  S:= Concat(S, DateSeparator);
        ^B:  S:= Concat(S, CurrencyString);
        ^C:  S:= Concat(S, ThousandSeparator);
        ^D:  S:= Concat(S, DecimalSeparator);
        ^E:  begin
               Inc(I);
               for J:= 1 to CurrencyDecimals do S:= Concat(S, P[N, I]);
             end;
        ^F:  S:= Concat(S, TimeSeparator);
        else S:= Concat(S, P[N, I]);     { or copy non-control chars }
      end;
    end;
  end else S:= '"unknown"';
  GetFmtStr:= S;
end;

{*****************************************************************}

{*
 * Prints the syntax.
 *}
procedure Usage;
begin
  Writeln('Usage: ', ExeName, ' [-h?L]');
  Writeln;
  Writeln('Options:');
  Writeln('  h, ?     display this help screen');
  Writeln('  L        display software licence');
  Halt(1);
end;

var
  I: Byte;
  Args: String;

begin
  Write(Copyright, ' ', Version, ', ');
  Writeln('Copyright (C) 1999-2002 Henrich Fukna');
  LicenceLine(Copyright, ExeName);
  Writeln;

  I:= 1;
  while I <= ParamCount do begin
    Args:= ParamStr(I);
    if (Args[1] = '-') then begin
      Delete(Args, 1, 1);
      while Length(Args) > 0 do begin
        case Args[1] of
          '?',
          'h': Usage;
          'L': begin
            Licence;
            Halt(1);
          end;
          else begin
            OptArgErrMsg(ExeName, False);
            Halt(2);
          end;
        end;
        Delete(Args, 1, 1);
      end;
    end else begin
      OptArgErrMsg(ExeName, True);
      Halt(2);
    end;
    Inc(I);
  end;

  Writeln('       Country code: ', CountryCode);
  Writeln('          Code page: ', CodePage);
  Writeln;
  Writeln('Thousands separator: "', ThousandSeparator, '"');
  Writeln('  Decimal separator: "', DecimalSeparator, '"');
  Writeln('     Date separator: "', DateSeparator, '"');
  Writeln('     Time separator: "', TimeSeparator, '"');
  Writeln('Data list separator: "', DataSeparator, '"');
  Writeln;
  Writeln('    Currency symbol: "', CurrencyString, '"');
  Writeln('    Currency digits: ', CurrencyDecimals);
  Writeln;
  Writeln('    Currency format: ', GetFmtStr(CurrencyFormat, 4, sCurrencyFormat));
  Writeln('        Date format: ', GetFmtStr(DateFormat, 2, sDateFormat));
  Writeln('        Time format: ', GetFmtStr(TimeFormat, 1, sTimeFormat));
  Writeln;
end.