{
 Country - The national application support for DOS.
 Copyright (C) 1998-2002  Henrich Fukna <fuky@azet.sk>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}
unit Dos_Intl;

{$IFNDEF MSDOS}
{$IFNDEF DPMI}
'"This must be compiled under DOS or DPMI compilation mode."'
{$ENDIF}
{$ENDIF}

interface
uses Dos, Strings;

var
  CountryCode: Word;                     { international country code }
  CodePage: Word;                        { DOS country code page }

  ThousandSeparator: Char;               { separator of thousands }
  DecimalSeparator: Char;                { separator of decimals }
  DateSeparator: Char;                   { date separator }
  TimeSeparator: Char;                   { time separator }
  DataSeparator: Char;                   { data list separator }

  CurrencyString: String;                { national currency string }
  CurrencyDecimals: Byte;                { number of significant decimal }
                                         { digits in currency notation }

  CurrencyFormat: Byte;                  { 0=$xx,1=xx$,2=$ xx,3=xx $,4=x$x }
  DateFormat: Byte;                      { 0=mdy, 1=dmy, 2=ymd }
  TimeFormat: Byte;                      { 0=12, 1=24 }

{*
 * Converts character to uppercase in currenty used alphabet.
 * (LATIN LETTER "c" WITH CARON -> LATIN LETTER "C" WITH CARON)
 *
 * @param C                     input character
 * @return upper character
 *}
function IntlUpCase(C: Char): Char;      { Casemap function }

{*****************************************************************}

implementation

type
  CountryBlock = record
    wDateFormat: Word;                   { 0=mdy, 1=dmy, 2=ymd }
    szCurrencySymb: array[0..4] of Char; { currency symbol     }
    szThouSep,                           { thousands separator }
    szDecSep,                            { decimal separator }
    szDateSep,                           { date separator }
    szTimeSep: array[0..1] of Char;      { time separator }
    bCurrencyFlags: Byte;                { 0=$xx,1=xx$,2=$ xx,3=xx $,4=x$x }
    bCurrencyDigits: Byte;               { decimals digits in currency }
    bTimeFormat: Byte;                   { 0=12, 1=24 }
    pfCasemap: Pointer;                  { Casemap FAR call address }
    szDataSep: array[0..1] of Char;      { data list separator }
    reserved: array[0..9] of Char;       { reserved }
  end;

var
  isIntl: Boolean;                       { country presence flag }
  St: String[5];
  Reg: Registers;                        { cpu registers }
  CIB: CountryBlock;                     { country information buffer }

{*
 * Converts character to uppercase in currenty used alphabet.
 * (LATIN LETTER "c" WITH CARON -> LATIN LETTER "C" WITH CARON)
 *
 * @param C                     input character
 * @return upper character
 *}
function IntlUpCase(C: Char): Char;
type
{$IFDEF DPMI}
  PtrRec = record
    Ofs, Seg: Word;
  end;

  RealRegs = record
    EDi, ESi, EBp, Dummy, EBx, EDx, ECx, EAx: LongInt;
    Flags, Es, Ds, Fs, Gs, Ip, Cs, Sp, Ss: Word;
  end;
{$ELSE}
  CaseMapFnc = function (C: Char): Char;
{$ENDIF}

begin
  if isIntl and (Ord(C) > 127) then      { for high 128 chars use }
  begin                                  { country-depend uppercase function }
{$IFDEF DPMI}
    RealRegs(RealmodeRegs).Eax:= Ord(C);
    RealRegs(RealmodeRegs).Cs:= PtrRec(CIB.pfCaseMap).Seg;
    RealRegs(RealmodeRegs).Ip:= PtrRec(CIB.pfCaseMap).Ofs;
    RealRegs(RealmodeRegs).Ss:= 0;
    RealRegs(RealmodeRegs).Sp:= 0;

    Reg.Ax:= $0301;
    Reg.Bx:= 0;
    Reg.Cx:= 0;
    Reg.Es:= Seg(RealmodeRegs);
    Reg.Di:= Ofs(RealmodeRegs);
    Intr($31, Reg);
    if (Reg.Flags and fCarry) = 0 then C:= Chr(RealRegs(RealmodeRegs).EAx)
    else C:= UpCase(C);
{$ELSE}
    C:= CaseMapFnc(CIB.pfCaseMap)(C);
{$ENDIF}
  end else
    C:= UpCase(C);                       { for first 128 chars use standard }
                                         { uppercase function }
  IntlUpCase:= C;
end;

{*****************************************************************}

{Initialition unit}

begin
{$IFOPT G+}
  if Test8086 = 0 then begin             { we are need 286+ }
    Writeln('This program requires i80286 or higher.');
    Halt(999);
  end;
{$ENDIF}
  if Swap(DosVersion) < $31E then begin  { we are need DOS 3.3+ }
    Writeln('This program requires MS-DOS 3.3 or higher.');
    Halt(998);
  end;

  isIntl:= False;
  Reg.Ah:= $38;                          { get country-depend info }
  Reg.Al:= 0;
  Reg.Ds:= Seg(CIB);
  Reg.Dx:= Ofs(CIB);
  MsDos(Reg);
  if (Reg.Flags and fCarry) = 0 then begin
    isIntl:= True;                       { if no error returned, set the }
                                         { country presence flag }
    CountryCode:= Reg.Ax;

    Reg.Ah:= $66;                        { get extended country information }
    Reg.Al:= 1;
    Reg.Bx:= CountryCode;
    MsDos(Reg);
    if (Reg.Flags and fCarry) = 0 then CodePage:= Reg.Bx;
  end;
  if not isIntl then begin               { if country not present }
    CountryCode:= 1;                     { use the default values }
    CodePage:= 437;

    CIB.wDateFormat:= 0;
    CIB.szCurrencySymb:= '$'#0#0#0#0;
    CIB.szThouSep:= ','#0;
    CIB.szDecSep:= '.'#0;
    CIB.szDateSep:= '-'#0;
    CIB.szTimeSep:= ':'#0;
    CIB.bCurrencyFlags:= 0;
    CIB.bCurrencyDigits:= 2;
    CIB.bTimeFormat:= 0;
    CIB.pfCasemap:= Ptr(0,0);
    CIB.szDataSep:= ','#0;
    CIB.reserved:= #0#0#0#0#0#0#0#0#0#0;
  end;
                                         { setting exported variables }
  St:= StrPas(CIB.szCurrencySymb); CurrencyString:=St;
  CurrencyFormat:= CIB.bCurrencyFlags;
  CurrencyDecimals:= CIB.bCurrencyDigits;

  St:= StrPas(CIB.szThouSep); ThousandSeparator:= St[1];
  St:= StrPas(CIB.szDecSep);  DecimalSeparator:= St[1];
  St:= StrPas(CIB.szDateSep); DateSeparator:= St[1];
  St:= StrPas(CIB.szTimeSep); TimeSeparator:= St[1];
  St:= StrPas(CIB.szDataSep); DataSeparator:= St[1];

  DateFormat:= CIB.wDateFormat;
  TimeFormat:= CIB.bTimeFormat;
end.